@@ -26,13 +26,13 @@
 #
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
-from optparse import OptionParser,SUPPRESS_HELP,IndentedHelpFormatter
+from optparse import OptionParser,SUPPRESS_HELP,IndentedHelpFormatter,Option,OptionValueError
 import textwrap
 import os.path
 import re
 import math
 import csv
-import copy
+from copy import copy
 import fnmatch
 import traceback
 import logging
@@ -113,6 +113,23 @@ class CommandParser:
          return "".join(result)
 
 
+   class CustomOption(Option):
+      def checkLongPositive(option,opt,value):
+         try:
+            longPositive = long(value)
+         except ValueError:
+            raise OptionValueError("option %s: invalid long positive value: %s" % (opt,str(value)))
+         else:
+            if longPositive < 1:
+               raise OptionValueError("option %s: invalid long positive value: %ld" % (opt,longPositive))
+            else:
+               return(longPositive)
+
+      TYPES = Option.TYPES + ("longPositive",)
+      TYPE_CHECKER = copy(Option.TYPE_CHECKER)
+      TYPE_CHECKER["longPositive"] = checkLongPositive
+
+
    def __grabMultipleStringArguments(self,option,opt_str,value,parser,isOptional):
       assert value is None
       value = getattr(parser.values,option.dest)
@@ -664,12 +681,11 @@ submit -p @@file=globnat:indeck* sim.exe @@file
 \n\n
 """
       indentedHelpFormatterWithNL = self.IndentedHelpFormatterWithNL()
-#     self.parser = OptionParser(prog="submit",add_help_option=False,
-#                                epilog=self.exampleDescriptions,formatter=indentedHelpFormatterWithNL)
       self.parser = OptionParser(prog="submit",add_help_option=False,
-                                 formatter=indentedHelpFormatterWithNL)
+                                 formatter=indentedHelpFormatterWithNL,
+                                 option_class=self.CustomOption)
       self.parser.disable_interspersed_args()
-      self.parser.add_option("-j","--jobid",action="store",type="long",dest="jobId",
+      self.parser.add_option("-j","--jobid",action="store",type="longPositive",dest="jobId",
                                             help=SUPPRESS_HELP)
 
       self.parser.set_defaults(help=False)
@@ -695,6 +711,10 @@ submit -p @@file=globnat:indeck* sim.exe @@file
       self.parser.set_defaults(localExecution=False)
       self.parser.add_option("-l","--local",action="store_true",dest="localExecution",
                                             help="Execute command locally")
+      self.parser.set_defaults(cacheId=0L)
+      self.parser.add_option("--cache",action="store",type="longPositive",dest="cacheId",
+                                       help=SUPPRESS_HELP)
+
       if doubleDashTerminator:
          self.parser.add_option("--status",action="callback",
                                            callback=self.__grabMultipleJobIdArguments,dest="statusIds",
@@ -785,7 +805,7 @@ submit -p @@file=globnat:indeck* sim.exe @@file
       self.parser.add_option("--detach",action="store_true",dest="detach",
                                         help="Detach client after launching run")
       self.parser.set_defaults(attachId=0L)
-      self.parser.add_option("--attach",action="store",type="long",dest="attachId",
+      self.parser.add_option("--attach",action="store",type="longPositive",dest="attachId",
                                         help="Attach to previously detached started server")
       self.parser.set_defaults(waitSwitch=False)
       self.parser.add_option("-W","--wait",action="store_true",dest="waitSwitch",
@@ -943,6 +963,7 @@ submit -p @@file=globnat:indeck* sim.exe @@file
    OPERATIONMODERUNDISTRIBUTOR     = 1 << 20
    OPERATIONMODERUNDISTRIBUTORID   = 1 << 21
    OPERATIONMODERUNVENUESTATUS     = 1 << 22
+   OPERATIONMODERUNCACHEHIT        = 1 << 23
 
 
    def getOperationMode(self):
@@ -975,6 +996,8 @@ submit -p @@file=globnat:indeck* sim.exe @@file
             operationMode = operationMode | self.OPERATIONMODERUNVENUESTATUS
          elif self.getOption('attachId') != 0L:
             operationMode = operationMode | self.OPERATIONMODERUNPROXY
+         elif self.getOption('cacheId') != 0L:
+            operationMode = operationMode | self.OPERATIONMODERUNCACHEHIT
          else:
             if   operationMode & (self.OPERATIONMODEHELPTOOLS | self.OPERATIONMODEHELPVENUES | \
                                   self.OPERATIONMODEHELPMANAGERS | self.OPERATIONMODEVERSIONDISTRIBUTOR):
@@ -1024,6 +1047,20 @@ submit -p @@file=globnat:indeck* sim.exe @@file
             self.logger.log(logging.ERROR,getLogMessage(errorMessage))
             validArguments = False
 
+      if self.getOption('cacheId'):
+         if self.getOption('localExecution'):
+            errorMessage = "Local execution cannot be combined with cache"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
+         if self.getOption('statusIds') != None:
+            errorMessage = "Status reporting cannot be combined with cache"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
+         if self.getOption('killIds') != None:
+            errorMessage = "Run termination cannot be combined with cache"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
+
       if self.getOption('help') or self.getOption('helpTools') or \
          self.getOption('helpVenues') or self.getOption('helpManagers') or self.getOption('helpExamples'):
          if self.getOption('detach'):
@@ -1034,6 +1071,10 @@ submit -p @@file=globnat:indeck* sim.exe @@file
             errorMessage = "Help cannot be combined with the attach option"
             self.logger.log(logging.ERROR,getLogMessage(errorMessage))
             validArguments = False
+         if self.getOption('cacheId'):
+            errorMessage = "Help cannot be combined with the cache option"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
 
       if self.getOption('version') or self.getOption('versionClient') or \
          self.getOption('versionServer') or self.getOption('versionDistributor'):
@@ -1045,6 +1086,10 @@ submit -p @@file=globnat:indeck* sim.exe @@file
             errorMessage = "version cannot be combined with the attach option"
             self.logger.log(logging.ERROR,getLogMessage(errorMessage))
             validArguments = False
+         if self.getOption('cacheId'):
+            errorMessage = "version cannot be combined with the cache option"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
 
       if self.getOption('runName'):
          runName = self.getOption('runName')
@@ -1087,6 +1132,10 @@ submit -p @@file=globnat:indeck* sim.exe @@file
             errorMessage = "Run attachment cannot be combined with run submission"
             self.logger.log(logging.ERROR,getLogMessage(errorMessage))
             validArguments = False
+         if self.getOption('cacheId') != 0L:
+            errorMessage = "cache cannot be combined with run submission"
+            self.logger.log(logging.ERROR,getLogMessage(errorMessage))
+            validArguments = False
 
       return(validArguments)
 
@@ -1209,12 +1258,16 @@ submit -p @@file=globnat:indeck* sim.exe @@file
          if dirname == "":
             dirname = os.getcwd()
          basename = os.path.basename(pattern)
-         dirFiles = os.listdir(dirname)
-         matchingFiles = fnmatch.filter(dirFiles,basename)
-         self.__natSort(matchingFiles)
-         parameterVector = []
-         for matchingFile in matchingFiles:
-            parameterVector.append(os.path.join(dirname,matchingFile))
+         try:
+            dirFiles = os.listdir(dirname)
+         except:
+            dirFiles = []
+         finally:
+            parameterVector = []
+            matchingFiles = fnmatch.filter(dirFiles,basename)
+            self.__natSort(matchingFiles)
+            for matchingFile in matchingFiles:
+               parameterVector.append(os.path.join(dirname,matchingFile))
       elif reGlobNatExpr.match(parameter):
       # look for files matching the globnat pattern
          pattern = reGlobNatExpr.match(parameter).group(1)
@@ -1222,12 +1275,16 @@ submit -p @@file=globnat:indeck* sim.exe @@file
          if dirname == "":
             dirname = os.getcwd()
          basename = os.path.basename(pattern)
-         dirFiles = os.listdir(dirname)
-         matchingFiles = fnmatch.filter(dirFiles,basename)
-         self.__natSort(matchingFiles)
-         parameterVector = []
-         for matchingFile in matchingFiles:
-            parameterVector.append(os.path.join(dirname,matchingFile))
+         try:
+            dirFiles = os.listdir(dirname)
+         except:
+            dirFiles = []
+         finally:
+            parameterVector = []
+            matchingFiles = fnmatch.filter(dirFiles,basename)
+            self.__natSort(matchingFiles)
+            for matchingFile in matchingFiles:
+               parameterVector.append(os.path.join(dirname,matchingFile))
       elif separator == "":
       # there is no separator char -- keep whole value string as one element
          parameterVector = []
